/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gen_define.h>
#include <co_canopen.h>

#include <gen_indices.h>

#include <stm32g4xx.h>
#include <stm32g4xx_hal.h>

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
#ifdef __GNUC__
  /* With GCC, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */
/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* CANopen LED defines */
#define LED_CANOPEN_GREEN_PORT GPIOC
#define LED_CANOPEN_GREEN_PIN GPIO_PIN_1
#define LED_CANOPEN_RED_PORT GPIOC
#define LED_CANOPEN_RED_PIN GPIO_PIN_0

/* user button defines */
#define BUTTON_PORT GPIOC
#define BUTTON_PIN GPIO_PIN_13

/* GPIO input defines */
#define DIGI_IN_1_PORT GPIOA
#define DIGI_IN_1_PIN GPIO_PIN_1
#define DIGI_IN_2_PORT GPIOA
#define DIGI_IN_2_PIN GPIO_PIN_0
#define DIGI_IN_3_PORT GPIOA
#define DIGI_IN_3_PIN GPIO_PIN_6
#define DIGI_IN_4_PORT GPIOA
#define DIGI_IN_4_PIN GPIO_PIN_7

/* running light defines */
#define LED_DIGI_OUT_ACTIVATED 1
#if defined (LED_DIGI_OUT_ACTIVATED)
#define LED_DIGI_OUT_1_PORT GPIOB
#define LED_DIGI_OUT_1_PIN GPIO_PIN_0
#define LED_DIGI_OUT_2_PORT GPIOA
#define LED_DIGI_OUT_2_PIN GPIO_PIN_4
#define LED_DIGI_OUT_3_PORT GPIOC
#define LED_DIGI_OUT_3_PIN GPIO_PIN_7
#define LED_DIGI_OUT_4_PORT GPIOA
#define LED_DIGI_OUT_4_PIN GPIO_PIN_9
#define LED_DIGI_OUT_5_PORT GPIOA
#define LED_DIGI_OUT_5_PIN GPIO_PIN_8
#define LED_DIGI_OUT_6_PORT GPIOB
#define LED_DIGI_OUT_6_PIN GPIO_PIN_10
#endif /* defined (LED_DIGI_OUT_ACTIVATED) */

#define DOMAIN_SIZE (10ul*1024ul)
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
FDCAN_HandleTypeDef hfdcan1;

UART_HandleTypeDef huart1;

/* USER CODE BEGIN PV */
static CO_TIMER_T simuApplTimer;
static CO_DOMAIN_PTR pDomainBuffer;
static UNSIGNED8 domainBuffer[DOMAIN_SIZE];
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_FDCAN1_Init(void);
static void MX_USART1_UART_Init(void);
/* USER CODE BEGIN PFP */
#if defined (LED_DIGI_OUT_ACTIVATED)
static RET_T objChangedInd(UNSIGNED16 index, UNSIGNED8 subindex);
#endif /* defined (LED_DIGI_OUT_ACTIVATED) */
static RET_T nmtInd(BOOL_T	execute, CO_NMT_STATE_T newState);
static void hbState(UNSIGNED8	nodeId, CO_ERRCTRL_T state,
		CO_NMT_STATE_T	nmtState);
static RET_T usdoServerReadInd(BOOL_T execute, UNSIGNED8	usdoClient,
		UNSIGNED16 index, UNSIGNED8	subIndex);
static RET_T usdoServerWriteInd(BOOL_T execute, UNSIGNED8 usdoClient,
		UNSIGNED16 index, UNSIGNED8	subIndex);
static void pdoInd(UNSIGNED16);
static void canInd(CO_CAN_STATE_T);
static void commInd(CO_COMM_STATE_EVENT_T);
static void ledGreenInd(BOOL_T);
static void ledRedInd(BOOL_T);
static void simuApp(void* pData);

extern void codrvTimerISR( void);
/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  BOOL_T useCANopenFD = CO_TRUE;
  UNSIGNED8	emcyData[5] = { 1u, 2u, 3u, 4u, 5u };
  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_FDCAN1_Init();
  MX_USART1_UART_Init();
  /* USER CODE BEGIN 2 */

  	/* Initialize the CAN FD controller with:
  	 * arbitration bitrate of 500 kBit/s
  	 * data bitrate of 2000 kBit/s */
  	if (codrvCanFdInit(500u, 2000u) != RET_OK)  {
  		Error_Handler();
  	}

  	/* provide a timer clock (systick) to the CANopen stack
  	 * this sets the systick intervall to CO_TIMER_INTERVAL */
  	if (codrvTimerSetup(CO_TIMER_INTERVAL) != RET_OK)  {
  		Error_Handler();
  	}

  	/* CANopen FD Initialization */

  	/* Initialize the CANopen stack in CANopen FD mode */
  	if (coCanOpenStackInit(NULL, &useCANopenFD) != RET_OK)  {
  		Error_Handler();
  	}

  	/* register event functions */

  	/* register NMT indication */
  	if (coEventRegister_NMT(nmtInd) != RET_OK)  {
  		Error_Handler();
  	}

  	/* register NMT error control indication */
  	if (coEventRegister_ERRCTRL(hbState) != RET_OK)  {
  		Error_Handler();
  	}

  	/* register CAN state indication */
  	if (coEventRegister_CAN_STATE(canInd) != RET_OK)  {
  		Error_Handler();
  	}

  	/* register communication event indication */
  	if (coEventRegister_COMM_EVENT(commInd) != RET_OK)  {
  		Error_Handler();
  	}

  	/* register USDO server read indication */
	if (coEventRegister_USDO_SERVER_READ(usdoServerReadInd) != RET_OK)  {
  		Error_Handler();
  	}

	/* register USDO server write indication */
	if (coEventRegister_USDO_SERVER_WRITE(usdoServerWriteInd) != RET_OK)  {
  		Error_Handler();
  	}

	/* register PDO indication (receive asynchronous) */
  	if (coEventRegister_PDO(pdoInd) != RET_OK)  {
  		Error_Handler();
  	}

  	/* register CANopen Run LED indication */
  	if (coEventRegister_LED_GREEN(ledGreenInd) != RET_OK)  {
  		Error_Handler();
  	}

  	/* register CANopen Error LED indication */
  	if (coEventRegister_LED_RED(ledRedInd) != RET_OK)  {
  		Error_Handler();
  	}

#if defined (LED_DIGI_OUT_ACTIVATED)
  	/* register CANopen Error LED indication */
  	if (coEventRegister_OBJECT_CHANGED(objChangedInd, 0x6200, 1u) != RET_OK)  {
  		Error_Handler();
  	}
#endif /* defined (LED_DIGI_OUT_ACTIVATED) */

  	/* enable CAN controller */
  	if (codrvCanEnable() != RET_OK)  {
  		Error_Handler();
  	}

  	/* start simulated application, cyclic every 100 milliseconds */
  	coTimerStart(&simuApplTimer, (100ul * 1000ul), simuApp, NULL, CO_TIMER_ATTR_ROUNDUP_CYCLIC);

  	/* initialize domain object */
  	pDomainBuffer = &domainBuffer[0];
  	coOdDomainAddrSet(I_MANUFACTURER_DATA, S_MANUFACTURER_DOMAIN, pDomainBuffer, DOMAIN_SIZE);

  	/* write Emergency Message to the Bus */
  	if (coEmcyWriteReq(0, 301, 0x9999, 1, 3, &emcyData[0]) != RET_OK)  {
  		Error_Handler();
  	}

  	/* turn all LEDs off */
    HAL_GPIO_WritePin(GPIOC, GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_7, GPIO_PIN_SET);
    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_4|GPIO_PIN_8|GPIO_PIN_9, GPIO_PIN_SET);
    HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0|GPIO_PIN_10, GPIO_PIN_SET);


    /* reconfigure GPIO input pins to pull up mode */
    GPIO_InitTypeDef GPIO_InitStruct = {0};
    GPIO_InitStruct.Pin = DIGI_IN_1_PIN|DIGI_IN_2_PIN|DIGI_IN_3_PIN|DIGI_IN_4_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
	  while (coCommTask() == CO_TRUE)  {
		  /* more CANopen FD messages have to be processed
		   * coCommTask() is executed again
		   */
	  }
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Configure the main internal regulator output voltage 
  */
  HAL_PWREx_ControlVoltageScaling(PWR_REGULATOR_VOLTAGE_SCALE1_BOOST);
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = RCC_PLLM_DIV3;
  RCC_OscInitStruct.PLL.PLLN = 40;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = RCC_PLLQ_DIV4;
  RCC_OscInitStruct.PLL.PLLR = RCC_PLLR_DIV2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_7) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the peripherals clocks 
  */
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1|RCC_PERIPHCLK_FDCAN;
  PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK2;
  PeriphClkInit.FdcanClockSelection = RCC_FDCANCLKSOURCE_PLL;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief FDCAN1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_FDCAN1_Init(void)
{

  /* USER CODE BEGIN FDCAN1_Init 0 */

  /* USER CODE END FDCAN1_Init 0 */

  /* USER CODE BEGIN FDCAN1_Init 1 */

  /* USER CODE END FDCAN1_Init 1 */
  hfdcan1.Instance = FDCAN1;
  hfdcan1.Init.ClockDivider = FDCAN_CLOCK_DIV1;
  hfdcan1.Init.FrameFormat = FDCAN_FRAME_FD_NO_BRS;
  hfdcan1.Init.Mode = FDCAN_MODE_NORMAL;
  hfdcan1.Init.AutoRetransmission = DISABLE;
  hfdcan1.Init.TransmitPause = DISABLE;
  hfdcan1.Init.ProtocolException = DISABLE;
  hfdcan1.Init.NominalPrescaler = 20;
  hfdcan1.Init.NominalSyncJumpWidth = 1;
  hfdcan1.Init.NominalTimeSeg1 = 13;
  hfdcan1.Init.NominalTimeSeg2 = 2;
  hfdcan1.Init.DataPrescaler = 1;
  hfdcan1.Init.DataSyncJumpWidth = 1;
  hfdcan1.Init.DataTimeSeg1 = 1;
  hfdcan1.Init.DataTimeSeg2 = 1;
  hfdcan1.Init.StdFiltersNbr = 0;
  hfdcan1.Init.ExtFiltersNbr = 0;
  hfdcan1.Init.TxFifoQueueMode = FDCAN_TX_FIFO_OPERATION;
  if (HAL_FDCAN_Init(&hfdcan1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN FDCAN1_Init 2 */

  /* USER CODE END FDCAN1_Init 2 */

}

/**
  * @brief USART1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART1_UART_Init(void)
{

  /* USER CODE BEGIN USART1_Init 0 */

  /* USER CODE END USART1_Init 0 */

  /* USER CODE BEGIN USART1_Init 1 */

  /* USER CODE END USART1_Init 1 */
  huart1.Instance = USART1;
  huart1.Init.BaudRate = 115200;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  huart1.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart1.Init.ClockPrescaler = UART_PRESCALER_DIV1;
  huart1.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_UARTEx_SetTxFifoThreshold(&huart1, UART_TXFIFO_THRESHOLD_1_8) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_UARTEx_SetRxFifoThreshold(&huart1, UART_RXFIFO_THRESHOLD_1_8) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_UARTEx_DisableFifoMode(&huart1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART1_Init 2 */

  /* USER CODE END USART1_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOF_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_7, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_4|GPIO_PIN_8|GPIO_PIN_9, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0|GPIO_PIN_10, GPIO_PIN_RESET);

  /*Configure GPIO pin : PC13 */
  GPIO_InitStruct.Pin = GPIO_PIN_13;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pins : PC0 PC1 PC7 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_7;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pins : PA0 PA1 PA6 PA7 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_6|GPIO_PIN_7;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PA4 PA8 PA9 */
  GPIO_InitStruct.Pin = GPIO_PIN_4|GPIO_PIN_8|GPIO_PIN_9;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PB0 PB10 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_10;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */
/*********************************************************************/
static void simuApp(void* pData)
{
UNSIGNED8 i;
INTEGER16 valAI;			/* analog input objects value */
UNSIGNED8 highestIndexAI;	/* analog input objects count */

	/* not used */
	(void)pData;

	/* simulate an application by increasing digital input and analog input values
	 * which are sent out via PDO */
	if (coOdGetObj_u8(I_READ_ANALOGUE_INPUT_16_BIT, 0, &highestIndexAI) == RET_OK)  {
		for (i = 1u; i <= highestIndexAI; i++)  {
			if (coOdGetObj_i16(I_READ_ANALOGUE_INPUT_16_BIT, i, &valAI) == RET_OK)  {
				valAI += i;
				if (coOdPutObj_i16(I_READ_ANALOGUE_INPUT_16_BIT, i, valAI) != RET_OK)  {
					printf("error increasing value!\n");
				}
			}
		}
	}

	/* read GPIO inputs and save state into the object dictionary*/
	if (HAL_GPIO_ReadPin(DIGI_IN_1_PORT, DIGI_IN_1_PIN) == GPIO_PIN_RESET)  {
		(void)coOdPutObj_u8(0x6000, 1u, 1u);
	} else {
		(void)coOdPutObj_u8(0x6000, 1u, 0u);
	}
	if (HAL_GPIO_ReadPin(DIGI_IN_2_PORT, DIGI_IN_2_PIN) == GPIO_PIN_RESET)  {
		(void)coOdPutObj_u8(0x6000, 2u, 1u);
	} else {
		(void)coOdPutObj_u8(0x6000, 2u, 0u);
	}
	if (HAL_GPIO_ReadPin(DIGI_IN_3_PORT, DIGI_IN_3_PIN) == GPIO_PIN_RESET)  {
		(void)coOdPutObj_u8(0x6000, 3u, 1u);
	} else {
		(void)coOdPutObj_u8(0x6000, 3u, 0u);
	}
	if (HAL_GPIO_ReadPin(DIGI_IN_4_PORT, DIGI_IN_4_PIN) == GPIO_PIN_RESET)  {
		(void)coOdPutObj_u8(0x6000, 4u, 1u);
	} else {
		(void)coOdPutObj_u8(0x6000, 4u, 0u);
	}
}


#if defined (LED_DIGI_OUT_ACTIVATED)
/*********************************************************************/
static RET_T objChangedInd(
		UNSIGNED16 index,
		UNSIGNED8 subindex
	)
{
UNSIGNED8 value;

	if ((index == 0x6200) && (subindex == 1u))  {
        coOdGetObj_u8(index, subindex, &value);

        HAL_GPIO_WritePin(LED_DIGI_OUT_1_PORT, LED_DIGI_OUT_1_PIN, (value & 1) == 0);
        HAL_GPIO_WritePin(LED_DIGI_OUT_2_PORT, LED_DIGI_OUT_2_PIN, (value & 2) == 0);
        HAL_GPIO_WritePin(LED_DIGI_OUT_3_PORT, LED_DIGI_OUT_3_PIN, (value & 4) == 0);
        HAL_GPIO_WritePin(LED_DIGI_OUT_4_PORT, LED_DIGI_OUT_4_PIN, (value & 8) == 0);
        HAL_GPIO_WritePin(LED_DIGI_OUT_5_PORT, LED_DIGI_OUT_5_PIN, (value & 16) == 0);
        HAL_GPIO_WritePin(LED_DIGI_OUT_6_PORT, LED_DIGI_OUT_6_PIN, (value & 32) == 0);
	}

	return(RET_OK);
}
#endif /* defined (LED_DIGI_OUT_ACTIVATED) */


/*********************************************************************/
static RET_T nmtInd(
		BOOL_T	execute,
		CO_NMT_STATE_T	newState
	)
{
	printf("nmtInd: New Nmt state %d - execute %d\n", newState, execute);

	return(RET_OK);
}


/*********************************************************************/
static void pdoInd(
		UNSIGNED16	pdoNr
	)
{
	printf("pdoInd: pdo %d received\n", pdoNr);
}


/*********************************************************************/
static void hbState(
		UNSIGNED8	nodeId,
		CO_ERRCTRL_T state,
		CO_NMT_STATE_T	nmtState
	)
{
	printf("hbInd: HB Event %d node %d nmtState: %d\n", state, nodeId, nmtState);

    return;
}

/*********************************************************************/
static RET_T usdoServerReadInd(
		BOOL_T		execute,
		UNSIGNED8	usdoClient,
		UNSIGNED16	index,
		UNSIGNED8	subIndex
	)
{
	printf("sdo server read ind: exec: %d, usdoClient %d, index %x:%d\n",
	execute, usdoClient, index, subIndex);

   // return(RET_INVALID_PARAMETER);
	return(RET_OK);
}


/*********************************************************************/
static RET_T usdoServerWriteInd(
		BOOL_T		execute,
		UNSIGNED8	usdoClient,
		UNSIGNED16	index,
		UNSIGNED8	subIndex
	)
{
	printf("sdo server write ind: exec: %d, usdoClient %d, index %x:%d\n",
	execute, usdoClient, index, subIndex);

   // return(RET_INVALID_PARAMETER);
	return(RET_OK);
}


/*********************************************************************/
static void canInd(
		CO_CAN_STATE_T	canState
	)
{
	switch (canState)  {
		case CO_CAN_STATE_BUS_OFF:
			printf("CAN: Bus Off\n");
			break;
		case CO_CAN_STATE_BUS_ON:
			printf("CAN: Bus On\n");
			break;
		case CO_CAN_STATE_PASSIVE:
			printf("CAN: Passive\n");
			break;
		default:
			break;
	}
}


/*********************************************************************/
static void commInd(
		CO_COMM_STATE_EVENT_T	commEvent
	)
{
	switch (commEvent)  {
		case CO_COMM_STATE_EVENT_CAN_OVERRUN:
			printf("COMM-Event: CAN Overrun\n");
			break;
		case CO_COMM_STATE_EVENT_REC_QUEUE_FULL:
			printf("COMM-Event: Rec Queue Full\n");
			break;
		case CO_COMM_STATE_EVENT_REC_QUEUE_OVERFLOW:
			printf("COMM-Event: Rec Queue Overflow\n");
			break;
		case CO_COMM_STATE_EVENT_REC_QUEUE_EMPTY:
			printf("COMM-Event: Rec Queue Empty\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_FULL:
			printf("COMM-Event: Tr Queue Full\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_OVERFLOW:
			printf("COMM-Event: Tr Queue Overflow\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_EMPTY:
			printf("COMM-Event: Tr Queue Empty\n");
			break;
		default:
			printf("COMM-Event: %d\n", commEvent);
			break;
	}

}

/*********************************************************************/
static void ledGreenInd(
		BOOL_T	on
	)
{
	/* toggle CANopen FD run LED */
	if (on == CO_TRUE) {
		HAL_GPIO_WritePin(LED_CANOPEN_GREEN_PORT, LED_CANOPEN_GREEN_PIN,  GPIO_PIN_RESET);
	} else {
		HAL_GPIO_WritePin(LED_CANOPEN_GREEN_PORT, LED_CANOPEN_GREEN_PIN,  GPIO_PIN_SET);
	}
}


/*********************************************************************/
static void ledRedInd(
		BOOL_T	on
	)
{
	/* toggle CANopen FD error LED */
	if (on == CO_TRUE) {
		HAL_GPIO_WritePin(LED_CANOPEN_RED_PORT, LED_CANOPEN_RED_PIN,  GPIO_PIN_RESET);
	} else {
		HAL_GPIO_WritePin(LED_CANOPEN_RED_PORT, LED_CANOPEN_RED_PIN,  GPIO_PIN_SET);
	}
}

/***************************************************************************/
/**
* HAL_SYSTICK_Callback - SYSTICK interrupt callback
*
*/
void HAL_SYSTICK_Callback(
		void	/* no parameter */
	)
{
	/* call timer interrupt service routine */
	codrvTimerISR();
}


/* printf() via UART1 */
int __io_putchar(int ch)
{
 uint8_t c[1];
 c[0] = ch & 0x00FF;
 HAL_UART_Transmit(&huart1, &*c, 1, 10);
 return ch;
}
int _write(int file,char *ptr, int len)
{
 int DataIdx;
 for(DataIdx= 0; DataIdx< len; DataIdx++)
 {
 __io_putchar(*ptr++);
 }
return len;
}


/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  while (1) {}
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
